<?php
/**
 * Plugin Name: Torknado Author
 * Description: Set up an author website as described in the book 'One-Hour Author Website' by Tyler Tork
 * Version:     20210627
 * Author:      Tyler Tork
 * Author URI:  https://torknado.com/
 * License:     GPL2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: torknado-auth
 * Domain Path: /languages
 *
 * Torknado Author is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * Torknado Author is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */
 
 // Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;
require_once(ABSPATH . 'wp-admin/includes/media.php');
require_once(ABSPATH . 'wp-admin/includes/file.php');
require_once(ABSPATH . 'wp-admin/includes/image.php');

/************************
 **  SHARED FUNCTIONS  **
 ************************/
if (!function_exists('torknado_add_dashboard_widget')) {
    function torknado_add_dashboard_widget( $widget_id, $widget_name, $callback, $control_callback = null, $callback_args = null ) {
	    // like the official function except the widget always goes to the top.
	    $screen = get_current_screen();
	    global $wp_dashboard_control_callbacks;
	 
	    $private_callback_args = array( '__widget_basename' => $widget_name );
	 
	    if ( is_null( $callback_args ) ) {
	        $callback_args = $private_callback_args;
	    } elseif ( is_array( $callback_args ) ) {
	        $callback_args = array_merge( $callback_args, $private_callback_args );
	    }
	 
	    if ( $control_callback && current_user_can( 'edit_dashboard' ) && is_callable( $control_callback ) ) {
	        $wp_dashboard_control_callbacks[ $widget_id ] = $control_callback;
	        if ( isset( $_GET['edit'] ) && $widget_id == $_GET['edit'] ) {
	            list($url)    = explode( '#', add_query_arg( 'edit', false ), 2 );
	            $widget_name .= ' <span class="postbox-title-action"><a href="' . esc_url( $url ) . '">' . __( 'Cancel' ) . '</a></span>';
	            $callback     = '_wp_dashboard_control_callback';
	        } else {
	            list($url)    = explode( '#', add_query_arg( 'edit', $widget_id ), 2 );
	            $widget_name .= ' <span class="postbox-title-action"><a href="' . esc_url( "$url#$widget_id" ) . '" class="edit-box open-box">' . __( 'Configure' ) . '</a></span>';
	        }
	    }
	 
	    $side_widgets = array( 'dashboard_quick_press', 'dashboard_primary' );
	 
	    $location = 'normal';
	    if ( in_array( $widget_id, $side_widgets ) ) {
	        $location = 'side';
	    }
	 
	    $priority = 'high';
	
	    add_meta_box( $widget_id, $widget_name, $callback, $screen, $location, $priority, $callback_args );
    }
    
	// retrieve the description of a menu location, e.g. input is "primary", returns "Main Menu"
    function torknado_get_menu_location_name($location) {
        $locs = get_registered_nav_menus(); // e.g. { "primary":"Main Menu", "secondary":"Secondary Menu" }
        return (isset($locs[$location]) ? $locs[$location] : false);
    }
	
	function torknado_find_role_like($role) {
		$locs = get_registered_nav_menus();
		if (isset($locs[$role]) && $locs[$role] != "") {
			return $role;
		} else { // didn't find the exact name, check whether the name is included in a different slug.
			foreach ($locs as $key => $value) {
				if (strpos($key, $location) !== false) {
					return $key;
				}
			}
		}
		return false;
	}
    
	// if we're on this plugin's "admin" page, load the stylesheet we use in the dashboard.
    function torknado_load_admin_scripts($hook) {
		if ($hook == 'toplevel_page_torknado-author' || $hook == 'appearance_page_tgmpa-install-plugins')
			wp_enqueue_style( 'torknado-styles', plugins_url('styles/torknado-admin.css', __FILE__) );
    }
    
    /* find or create a menu that has a specified location assigned in the theme. menu_role is e.g. "primary", default_name is
	   human-readable description, default_slug is slug of menu. */
    function torknado_create_menu_at_location($menu_role, $default_name, $default_slug) {
		$exact_role = torknado_find_role_like($menu_role);
		if ($exact_role === false) {
	    	printf('<p>%s</p>', esc_html(sprintf(__('This theme does not support use of a "%s" menu.'), $menu_role)));
			if ($torknado_debug) echo "<p>get_registered_nav_menus = " . var_dump(get_registered_nav_menus()) . "</p>";
	    	return null;
	    }
		//@@echo "<p>exact role = " . $exact_role . "</p>";
		$role_name = torknado_get_menu_location_name($exact_role);
		//@@echo "<p>role_name = " . $role_name . "</p>";
	    $locations = get_nav_menu_locations(); // menu role is key for menu ID, e.g. {'primary':2}
		//@@echo "<p>locations = " . esc_html(serialize($locations)) . "</p>";
	    $menu_id = $locations[ $exact_role ];
	    if ($menu_id == "") {
	        // there's no menu in the role we want -- search for one with the slug we expect.
	        //@@echo "<p>no menu in location = " . $exact_role . "</p>";
	        $menu_id = $default_slug;
	        $bAssign = true;
	    } else { // a menu already is in that location, so don't assign a new one there.
	        $bAssign = false;
	    }
	    $menu = wp_get_nav_menu_object($menu_id);
	    if ($menu === false) {
	        // create a new menu using default parameters from caller.
	        //wp_update_nav_menu_object( 0, array('menu-name' => $default_name, '' ) );
	        $menu_id = wp_create_nav_menu($default_slug);
	        wp_update_nav_menu_object( $menu_id, array('menu-name' => $default_name) );
	        echo '<p>' . esc_html(sprintf(__('Created menu "%s".'), $default_name)) .  '</p>';
	        $menu = wp_get_nav_menu_object($menu_id );
	        $our_menu_slug = $menu->slug;
	    }
	    // whatever menu we found or created, if it doesn't already have the desired role, assign it now.
	    if ($bAssign) {
	        printf('<p>%s</p>', esc_html(sprintf(__('Assigned menu "%s" as the new %s.'), $menu->name, $role_name)));
	        //@@printf("<p>menu=%s</p>", serialize($menu));
	        $locations[$exact_role] = $menu->term_id;
	        set_theme_mod( 'nav_menu_locations', $locations );
	    } else {
	        printf('<p>%s</p>', esc_html(sprintf(__('A "%s" menu already exists.'), $default_name)));
	    }
	    return $menu;
    }

    function torknado_populate_menu($menu, $new_items) {
    	// Create menu items specified by caller, if they don't exist already. Return true if any change was made.
    	if ($menu === null) return false;
    	$rval = false;
    	$allitems = wp_get_nav_menu_items($menu);
    	foreach ($new_items as $new_item) {
            if ($new_item['menu-item-object'] == '') $new_item['menu-item-object'] = 'custom';
            $pos = -1;
            $issame = false;
            // check whether the menu item already exists, don't make a duplicate.
            if (is_array($allitems)) {
                foreach ($allitems as $key => $an_item) {
                    if ($an_item->object == $new_item['menu-item-object']) {
                    	if ($an_item->object == 'custom') {
                            $issame = ($an_item->url == $new_item['menu-item-url']);
                    	} else {
                            $issame = ($an_item->object_id == $new_item['menu-item-object-id']);
                    	}
                    }
                    if ($issame) {
                    	$pos = $key;
                    	break;
                    }
            	}
            }
            if ($pos < 0) {
//@@ echo '<p>create menu item</p>';
		$rval = true;
        	$new_item['menu-item-status'] = 'publish';
        	wp_update_nav_menu_item($menu->term_id, 0, $new_item);
            }
	}
        return $rval;
    }

    function torknado_strleftback($haystack, $needle) {
        $pos = strrpos($haystack, $needle);
        return $pos === false ? '' : substr($haystack, 0, $pos);
    }
	
	function torknado_get_remote_image($url, $slug) {
		$page = get_page_by_path( $slug, '', 'attachment' );
		if ($page == null) {
			$newId = media_sideload_image( $url, null, $slug, 'id');
			return $newId;
		} else {
			return $page->ID;
		}
	}
	
	function torknado_striendswith(string $haystack , string $needle) {
		return strcasecmp($needle, substr($haystack, -strlen($needle))) == 0;
	}
}
/****************************
 **  END SHARED FUNCTIONS  **
 ****************************/

function torknado_author_show_dash_widget() {
    // provides content for a dashboard widget -- just give a link to our Tools page.
    printf(esc_html__('Visit %s to run the author website setup tool.'), '<a href="/wp-admin/admin.php?page=torknado-author">' . __('Author Site Setup') . '</a>');
}

function torknado_author_plugin_menu() {
	add_menu_page(
		__('Author Site Setup'), // page title
		__('Author Site Setup'), // menu title
		'manage_options',
		'torknado-author',
		'torknado_author_admin_page',
		plugin_dir_url(__FILE__) . 'tornado-menu-icon.png',
		1
	);
 //   add_management_page( 'Author Site Setup', 'Author Site Setup', 'install_plugins', 'torknado-author', 'torknado_author_admin_page' );
}

function torknado_author_remove() {
    deactivate_plugins( plugin_basename( __FILE__ ) );
    echo 'Plugin disabled; <a href="/wp-admin/index.php">return to dashboard</a>.';
}

function torknado_menu_link_intext() {
	return '<span style=\'color:white;background-color:#23282d; display:inline-block; padding:4px\'><img src=\'/wp-content/plugins/torknado-author/tornado-menu-icon.png> ' . __('Author Site Setup') . '</span>';
}

function torknado_author_admin_page() {
// render the Tools page to install the Author site elements.
    wp_register_style( 'torknado-style', plugins_url( '/styles/torknado-admin.css', __FILE__ ));
    echo '<div class="torknado-admin-page"><h1>' . esc_html__('One Hour Author Website Setup') . '</h1>';
    if (!torknado_author_required_plugins_active()) {
?>
<p>This screen is displayed by the <strong>Torknado Author</strong> setup tool. Above, there should be a message listing plugins that can be
automatically installed. Some of these must be installed before the automatic website setup can occur.
Click <strong>Begin installing plugins</strong> above to begin this process.</p>
<p>Some plugins are optional; read the comments to decide which you need for your site. Once all 'Required' plugins are installed, further
information will appear on this screen.</p>
<p>To return to this screen, use the
<span style='color:white;background-color:#23282d; display:inline-block; padding:4px'><img src='/wp-content/plugins/torknado-author/tornado-menu-icon.png'> Author Site Setup</span>
link at the top left of the dashboard.<?php
	} else if($_SERVER['REQUEST_METHOD'] == "POST") {
        if (isset($_POST['torknado-author-setup'])) {
            torknado_author_setup_site();
            torknado_author_show_buttons();
        } else if (isset($_POST['torknado-author-remove'])) {
            torknado_author_remove();
        }
    } else {
?>
<p>This screen is displayed by the <strong>Torknado Author</strong> setup tool. If any suggested plugins are still listed above, they are optional.
Automatic setup can proceed.</p>
<p>The <strong>Setup</strong> button below will automatically set appropriate options and  create pages and menus needed for the site. Using the button more than once will not cause a problem.</p>
<div class='updated'><p><strong>FIRST</strong>, please confirm your website title (<em><?php echo get_option('blogname') ?></em>) is correct. If not, go to <a href='/wp-admin/options-general.php'>Settings &gt; General</a> now and correct it, then click <strong>Author Site Setup</strong> in the dashboard menu to return here.</p></div>
<p>The button does the following, as needed:
<ul>
<li>Create a contact form (to appear on your <strong>Contact</strong> page).</li>
<li>Create a subscriber mailing list and signup form.</li>
<li>Create the pages <strong>My Books</strong>, <strong>Blog</strong>, <strong>About</strong>, <strong>Contact</strong>, and <strong>Subscribe</strong>.</li>
<li>Make <strong>My Books</strong> the landing page.</li>
<li>Create a primary menu.</li>
<li>Add menu entries for the above pages.</li>
<li>If the theme supports it, create a "social links" menu, which you'll customize with links to your social media profile pages.</li>
<li>If the theme has a suitable seeming widget area, add sidebar widgets, including one linking to the Subscribe page.</li>
<li>Set thumbnails to not crop to squares, and set "medium" image size to 350x400.</li>
</ul><?php
if (torknado_get_menu_location_name('social') === false) {
    $theme = wp_get_theme();
    $themeName = esc_html($theme->get('Name'));
    printf( '<div class="updated"><p>%s</p></div>', sprintf(esc_html__( 'WARNING: This theme, "%s", does not support a "social" menu, so your links to your social media pages will not appear. Instead, use the "Sassy Social Share - Follow Icons" widget.' ), $themeName));
}
?><p>Once you finish setting up your site, click the <strong>Deactivate</strong> button to remove this screen from your dashboard (or deactivate and delete the <strong>Torknado Author</strong> plugin from the Plugins screen). Doing this will not undo anything this tool has done.
<?php
	torknado_author_show_buttons();
	?><p>After you click Setup, visit the <a href="https://torknado.com/auth-setup" target="nado-checklist">setup instructions page</a> (which is probably where you downloaded this plugin) for any corrections or added instructions.</p>
<p>You can switch themes while this tool is still active. If doing so messes anything up, use the Setup button again to correct it.</p><?php
		echo '</div>';
    }
}

function torknado_author_show_buttons() {
    echo '<div class="torknado-button-area"><div style="width: 30%; display: inline-block">';
    echo '<form action="' . admin_url( 'admin.php?page=torknado-author') . '" method="post"><input type="submit" class="button-primary" name="torknado-author-setup" value="' . esc_html__('Setup') . '" /></form>';
    echo '</div><div style="display: inline-block;">';
    echo '<form action="' . admin_url( 'admin.php?page=torknado-author') . '" method="post"><input type="submit" class="button-primary" name="torknado-author-remove" value="' . esc_html__('Deactivate') . '" /></form>';
    echo '</div></div>';
}

function torknado_author_setup_media_settings() {
    // image settings
    // do not make thumbnails square.
    $bSet = false;
    if (update_option('thumbnail_crop', 0)) {
    	$bSet = true;
    	printf('<p>%s</p>', esc_html__('Turn off thumbnail cropping.'));
    }
    if (update_option('medium_size_w', 350 ) or update_option('medium_size_h', 400 )) {
    	$bSet = true;
    	printf('<p>%s</p>', esc_html__('Set medium image width to 350x400.'));
    }
    if (!$bSet){
	printf('<p>%s</p>', esc_html__('No change needed to media settings.'));
    }
}

function torknado_get_igform_id($form_name) {
	$obj = new ES_DB_Forms();
	$forms = $obj->get_forms_id_name_map();
	foreach ($forms as $id => $aName) {
		if ($aName == $form_name) {
			return $id;
		}
	}
	return 0;
}

function torknado_create_iglist($silent = false) {
	$list_name = __('Main');
	$objL = new ES_DB_Lists();
	$default_list = $objL->get_list_by_name( $list_name );
	$id = $default_list['id'];
	if ($id == false) {
		$all = $objL->get_lists();
		if (count($all) > 0) {
			$id = $all[0]['id'];
		}
	}
	if ($id != false) {
		if (!$silent) echo '<p>' . esc_html__('Subscriber mailing list already exists') . '</p>';
		return $id;
	}
	$id = $objL->add_list($list_name, 'main');
	echo '<p>' . esc_html__(sprintf('Created subscriber mailing list \"%s\".', $list_name)) . '</p>';
	return $id;
}

function torknado_set_option($optname, $value, $notice = NULL) {
	$cur = get_option($optname);
	if ($cur != $value) {
		if ($value === "") {
			delete_option($optname);
		} else {
			update_option($optname, $value, $autoload);
		}
		if (!empty($notice) && $notice != "") {
			echo '<p>' . esc_html($notice). '</p>';
		}
		return true;
	} else {
		return false;
	}
}

function torknado_set_option_list($opts, $autoload = false) {
	$mod = false;
	foreach ($opts as $optname => $rec) {
		$cur = get_option($optname);
		$value = $rec['value'];
		$matchexpr = $rec['match'];
		if (empty($matchexpr)) {
			$match = ($cur !== $value);
		} else if ($matchexpr[0] == '/') {
			$match = preg_match($matchexpr, maybe_serialize($cur));
		} else {
			$match = (strpos($cur, $rec['match']) !== false);
		}
		if ($match) {
			if ($value === "") {
				delete_option($optname);
			} else {
				update_option($optname, $value, $autoload);
			}
			$mod = true;
			if (!empty($rec['message'])) {
				echo '<p>' . esc_html($rec['message']). '</p>';
			}
		}
	}
	return $mod;
}

function torknado_set_ig_options() {
	$opts = array(
		'ig_es_from_name' => array('value' => get_option('blogname')),
		'ig_es_admin_new_contact_email_subject' => array('value' => 'New newsletter subscriber'),
		'ig_es_admin_new_contact_email_content' => array('value' => '{{NAME}} ({{EMAIL}}) joined {{LIST}} list(s).'),
		'ig_es_confirmation_mail_subject' => array('value' => 'Please confirm subscription to {{SITENAME}}'),
		'ig_es_confirmation_mail_content' => array('value' => 'Someone signed you up for email updates from {{SITENAME}}. If it was you, please <a href=\'{{SUBSCRIBE-LINK}}\'>click this link to confirm your subscription</a>.

Otherwise, you can ignore this message (sorry to bother you).

Thanks!'),
		'ig_es_welcome_email_content' => array('value' => 'Hi {{NAME}},

Thank you for confirming your subscription to updates from {{SITENAME}}!

I\'ll try to keep it interesting!'),
		'ig_es_unsubscribe_link_content' => array('value' => 'If you no longer wish to receive these emails, <a href=\'{{UNSUBSCRIBE-LINK}}\'>unsubscribe here</a>.', 'match' => '>here<'),
		'ig_es_form_submission_success_message' => array('value' => 'Please check your email for a confirmation link. If it doesn\'t arrive within a few minutes, check the junk/spam folder.', 'match' => 'Kindly'),
		'ig_es_powered_by' => array('value' => 'no'),
		'ig_es_subscription_error_messsage' => array('match' => 'Something\'s amiss', 'value' => 'Your email address could not be added. Usually this is because the address is already subscribed.'),
		'ig_es_unsubscribe_error_message' => array('match' => 'Urrgh', 'value' => 'Unsubscribe failed. Usually this is because the email address is not a subscriber. If you have a problem still, please use our <a href=\'/contact\'>Contact form</a> to request assistance. We really <strong>don\'t</strong> want to send you unwanted email.'),
	);
	
	if (torknado_set_option_list($opts)) {
		echo '<p>' . esc_html__('Updated some Email Subscribers plugin options.') . '</p>';
	} else {
		echo '<p>' . esc_html__('No changes to Email Subscribers plugin options.') . '</p>';
	}
}

function torknado_post_setup_options() {
	$opts = array(
			'_transient_timeout_heateor-sss-admin-notice-on-activation' => array('value' => ''),
			'_transient_heateor-sss-admin-notice-on-activation' => array('value' => ''),
			'wp_add_mime_types_array' => array(
				'value' => array ( 'mime_type_values' => 'a:4:{i:0;s:27:"epub = application/epub+zip";i:1;s:37:"mobi = application/x-mobipocket-ebook";i:2;s:39:"azw|azw3 = application/vnd.amazon.ebook";i:3;s:0:"";}', 'security_attempt_enable' => 'no', 'filename_sanitized_enable' => 'no', 'file_type_debug' => 'no'),
				'match' => '/^((?!vnd\\.amazon\\.ebook).)*$/',
				'message' => 'Set up to allow e-Book uploads.'
				)
		);
	torknado_set_option_list($opts);
}

/* this function contains options we want to set before installing plugins, mostly for the purpose
   of preemptively dismissing those plugins' welcome and nag notices. My users have a mission and
   I want to minimize distractions.
*/
function torknado_set_preinstall_options() {
	$opts = array(
			'ig_es_onboarding_complete' => array('value'=>'yes')
		);
	torknado_set_option_list($opts);
}

function torknado_create_igform() {
	
	$from_email = get_option( 'ig_es_from_email', true );
	$mydomain = $_SERVER['HTTP_HOST'];
	if (!torknado_striendswith($from_email, '@' . $mydomain)) {
		$from_email = 'noreply@' . $mydomain;
		update_option( 'ig_es_from_email', $from_email );
		echo '<p>' . sprintf(esc_html__('Changed "from" address for newsletter subscriptions to "%s".'), $from_email) . '</p>';
	}
	torknado_set_ig_options();
	$form_name = 'Full Screen Subscribe';
	$form_id = torknado_get_igform_id($form_name);
	if ($form_id > 0) {
		echo '<p>' . esc_html__('Subscription form already exists.') . '</p>';
		return $form_id;
	}
	
	$form_data = array();
	$objF = new ES_DB_Forms();
	$list_id      = torknado_create_iglist();
	
	$body = array(
		array(
			'type' => 'text',
			'name'   => 'Name',
			'id'     => 'name',
			'params' => array(
				'label'    => 'Name (optional)',
				'show'     => true,
				'required' => false
			),
			'position' => 1
		),
		array(
			'type'   => 'text',
			'name'   => 'Email',
			'id'     => 'email',
			'params' => array(
				'label'    => 'Email address',
				'show'     => true,
				'required' => true
			),
			'position' => 2
		),
		array(
			'type'   => 'checkbox',
			'name'   => 'Lists',
			'id'     => 'lists',
			'params' => array(
				'label'    => 'Lists',
				'show'     => false,
				'required' => true,
				'values' => array($list_id)
			),
			'position' => 3
		),
		
		array(
			'type'   => 'submit',
			'name'   => 'submit',
			'id'     => 'submit',
			'params' => array(
				'label' => 'Subscribe',
				'show'  => true
			),
			'position' => 5
		),
	);
	
	$settings = array(
		'lists' => array( $list_id ),
		'desc'  => 'Enter your information here to receive updates from this site.',
		'form_version' => '1.0',
		'captcha' => 'no',
		'gdpr' => array(
			'consent' => 'yes',
			'consent_text' => 'I consent to this information being stored for the purpose of sending email newsletters pertaining to this website. I understand the information will not be used for other purposes or shared with third parties, except insofar as may be necessary to accomplish the above stated purpose.'
		)
	);
	
	$form_data['name'] = $form_name;
	$form_data['body']       = maybe_serialize( $body );
	$form_data['settings']   = maybe_serialize( $settings );
	$form_data['styles']     = '';
	$form_data['created_at'] = ig_get_current_date_time();
	$form_data['updated_at'] = null;
	$form_data['deleted_at'] = null;
	$form_data['af_id']      = 0;
	
	$form_id = $objF->add_form( $form_data );
	echo '<p>' . esc_html__(sprintf('Created subscription form "%s".', $form_name)) . '</p>';
	return $form_id;
}

function torknado_create_igcampaign() {
	//@@@
	$postobj = get_page_by_path( 'blog-update', '', 'es_template' );
	if( $postobj === null ) return; // was supposed to have been created when creating pages.
	$post_id = $postobj->ID;
	$list_id = torknado_create_iglist(true);
	$objC = new ES_DB_Campaigns();
	$count = $objC->get_total_post_notifications();
	if ($count > 0) return; // there's already a post notification set up.
	$mydomain = $_SERVER['HTTP_HOST'];
	$data = array(
		'slug' => 'new-post',
		'name' => __('New Post Notification'),
		'type' => 'post_notification',
		'from_email' => 'noreply@' . $mydomain,
		'reply_to_email' => 'noreply@' . $mydomain,
		'from_name' => get_option('blogname'),
		'reply_to_name' => get_option('blogname'),
		'categories' => '##{a}All{a}##',
		'list_ids' => $list_id,
		'base_template_id' => $post_id,
		'status' => 1
		);
	$broadcast_id = $objC->save_campaign( $data );
	echo sprintf('<p>%s</p>', __('Created automatic blog post notification'));
}

function torknado_author_setup_share_options() {
    // for Sassy share buttons plugin, set options.
    $bSet = false;
    $tmp = get_option('heateor_sss');
    if ($tmp === false or empty($tmp)) {
		printf('<p>%s</p>', esc_html__('Sassy Social Share has not been activated; skipping Social settings.'));
		return;
    }
	if (!empty($tmp["vertical_enable"])) { // turn off floating buttons for now.
		unset($tmp["vertical_enable"]);
		$bSet = true;
	}
	$showEm = array('post', 'page', 'excerpt', 'category', 'mbdb_book', 'archive');
	foreach ($showEm as $pagetype) {
		if (empty($tmp[$pagetype])) {
			$tmp[$pagetype] = "1";
			$bSet = true;
		}
	}
	if ($tmp['title'] == __('Spread the love')) {
		$tmp['title'] = __('Share');
		$bSet = true;
	}
    if ($bSet) {
		update_option('heateor_sss', $tmp);
    	printf('<p>%s</p>', esc_html__('Set options for Sassy Social Share.'));
    } else {
	printf('<p>%s</p>', esc_html__('No change needed to Sassy Social Share options.'));
    }
}

function torknado_author_create_pages() {
    // create default pages.
	
	$sample_featured_image = torknado_get_remote_image('https://torknado.com/wp-content/uploads/2021/01/featured-image-square.png', 'sample-featured-image');
	$sample_embedded_image = torknado_get_remote_image('https://torknado.com/wp-content/uploads/2021/01/sample-embedded.png', 'sample-embedded-image');
	$sample_embedded_attach = get_post($sample_embedded_image);
	
	$subscribe_form_id = torknado_create_igform();
	
    $default_pages = array(
        array(
            'title' => 'All',
            'slug' => 'all',
            'type' => 'mbdb_book_grid',
            'meta' => '_mbdb_book_grid_books=all,_mbdb_book_grid_group_by_level_1=none,_mbdb_book_grid_group_by_level_2=none,_mbdb_book_grid_group_by_level_3=none,_mbdb_book_grid_group_by_level_4=none,_mbdb_book_grid_group_by_level_5=none,_mbdb_book_grid_group_by_level_6=none,_mbdb_book_grid_group_by_level_7=none,_mbdb_book_grid_order=pubdateA,_mbdb_book_grid_cover_height_default=yes'
            ),
        array(
            'title' => __('My Books'),
            'slug' => 'books',
            'content' => '<!-- wp:paragraph --><p>'.__('TODO: Create Book pages for each of your books. The covers will automatically appear below. Change this text to suit yourself.').'</p><!-- /wp:paragraph --><!-- wp:shortcode -->[mbm_book_grid id="<lastid>"]<!-- /wp:shortcode --><!-- wp:paragraph --><p>'.__('TODO: Edit or delete this text which appears below your book grid.').'</p><!-- /wp:paragraph -->'
            ),
        array(
            'title' => __('Blog'),
            'slug' => 'blog',
            'content' => 'Do not edit this text; blog entries will appear in this space.'
            ),
        array(
            'title' => __('About'),
            'slug' => 'about',
            'content' => '<!-- wp:paragraph --><p>' . __('TODO: edit page title, add your photo and bio.') . '</p><!-- /wp:paragraph -->'
            ),
		array(
			'title' => __('First Contact'),
			'slug' => 'first-contact',
			'type' => 'wpforms',
			'content' => '{"fields":[{"id":"0","type":"name","label":"Name","format":"first-last","description":"","required":"1","size":"medium","simple_placeholder":"","simple_default":"","first_placeholder":"","first_default":"","middle_placeholder":"","middle_default":"","last_placeholder":"","last_default":"","css":""},{"id":"1","type":"email","label":"Email","description":"","required":"1","size":"medium","placeholder":"","confirmation_placeholder":"","default_value":"","css":"","filter_type":"","allowlist":"","denylist":""},{"id":"2","type":"textarea","label":"Comment or Message","description":"","required":"1","size":"medium","placeholder":"","limit_count":"1","limit_mode":"characters","default_value":"","css":""}'
/* . ',{"id":"3","type":"text","label":"Spambot test","description":"Please type &quot;human&quot; in this field if you\'re a human being, or something else if you\'re an evil spambot. Answer wrong and your message WILL be discarded.","required":"1","size":"medium","placeholder":"","limit_count":"1","limit_mode":"characters","default_value":"","css":"","input_mask":""}' */
			. '],"id":"<ownid>","field_id":4,"settings":{"form_title":"First Contact","form_desc":"","form_class":"","submit_text":"Submit","submit_text_processing":"Sending...","submit_class":"","antispam":"1","notification_enable":"1","notifications":{"1":{"email":"{admin_email}","subject":"New website contact request","sender_name":"Website Contact Form","sender_address":"wordpress@' . $_SERVER['HTTP_HOST'] . '","replyto":"{field_id=\\\\"1\\\\"}","message":"{all_fields}"}},"confirmations":{"1":{"type":"message","message":"<p>Thanks for contacting us! We will be in touch with you shortly.<\\\\/p>","message_scroll":"1","page":"50","redirect":""}}},"meta":{"template":"contact"}}'),
        array(
            'title' => __('Contact'),
            'slug' => 'contact',
            'content' => '<!-- wp:paragraph --><p>TODO: Edit this text. If you don\'t want a contact form, delete the contact form block below.</p><!-- /wp:paragraph --><!-- wp:wpforms/form-selector {"formId":"<lastid>"} /-->',
			'meta_input' => array('_heateor_sss_meta'=>array('sharing'=>'1', 'vertical_sharing'=>'1')),
			'searchExclude' => true
            ),
		array(
			'title' => __('Subscribe'),
			'slug' => 'subscribe',
			'content' => '<!-- wp:paragraph --><p>TODO: Customize text to accurately describe what email subscribers can expect to receive.</p><!-- /wp:paragraph -->
<!-- wp:paragraph --><p>Subscribers to my email list will receive</p><!-- /wp:paragraph -->
<!-- wp:list -->
<ul><li>an email notification whenever I publish a blog post</li><li>occasional (not more than 4 per year) newsletters about my life, opinions, and forthcoming publications</li><li>occasional freebies or contests <strong>from me</strong>.</li></ul>
<!-- /wp:list -->
<!-- wp:paragraph --><p>Your information will not be sold or shared, but may be stored on the servers of a mailing service (such as Mailchimp). You can cancel at any time and your information will be removed.</p><!-- /wp:paragraph -->
<!-- wp:shortcode -->[email-subscribers-form id="' . $subscribe_form_id . '"]<!-- /wp:shortcode -->'
		),
		array(
			'slug' => 'styling-test',
			'title' => 'Theme Styling Test',
			'content' => '<!-- wp:columns -->
<div class="wp-block-columns"><!-- wp:column -->
<div class="wp-block-column"><!-- wp:heading -->
<h2>Heading level 2, which is the highest you should use</h2>
<!-- /wp:heading -->

<!-- wp:heading {"level":3} -->
<h3>Heading level 3</h3>
<!-- /wp:heading -->

<!-- wp:heading {"level":4} -->
<h4>Heading level 4</h4>
<!-- /wp:heading -->

<!-- wp:heading {"level":5} -->
<h5>Heading level 5</h5>
<!-- /wp:heading -->

<!-- wp:heading {"level":6} -->
<h6>Heading level 6</h6>
<!-- /wp:heading -->

<!-- wp:image {"id":' . $sample_embedded_image . ',"sizeSlug":"large","linkDestination":"none"} -->
<figure class="wp-block-image size-large"><img src="' . $sample_embedded_attach->guid . '" alt="" class="wp-image-' . $sample_embedded_image . '"/></figure>
<!-- /wp:image --></div>
<!-- /wp:column -->

<!-- wp:column -->
<div class="wp-block-column"><!-- wp:paragraph -->
<p>This is a paragraph of normal text. I\'m making the line long enough to wrap so you can see the interline spacing.</p>
<!-- /wp:paragraph -->

<!-- wp:paragraph -->
<p>Here\'s another paragraph preceding a bullet list.</p>
<!-- /wp:paragraph -->

<!-- wp:list -->
<ul><li>First item in bullet list is long so you can see how the list wraps.</li><li>Second item, with subitems following.<ul><li>Subordinate, indented item, also long enough to wrap.</li><li>More...<ul><li>Another level of indentation</li><li>That\'s enough indentation</li></ul></li><li>Out</li></ul></li><li>Out</li></ul>
<!-- /wp:list -->

<!-- wp:paragraph -->
<p>A brief paragraph following the list.</p>
<!-- /wp:paragraph --></div>
<!-- /wp:column --></div>
<!-- /wp:columns -->

<!-- wp:paragraph -->
<p>What about numbered lists?</p>
<!-- /wp:paragraph -->

<!-- wp:list {"ordered":true} -->
<ol><li>The first item, which is long enough to wrap, not that the line spacing is likely to be different from that of bullet lists.</li><li>Second item.<ol><li>Addendum to second item.</li><li>Further thoughts on item 2.</li></ol></li><li>And what about the third item?</li></ol>
<!-- /wp:list -->

<!-- wp:paragraph -->
<p>How about block quotes?</p>
<!-- /wp:paragraph -->

<!-- wp:quote -->
<blockquote class="wp-block-quote"><p>This is a wise saying someone once said.</p><cite>Person who said it.</cite></blockquote>
<!-- /wp:quote -->

<!-- wp:paragraph -->
<p>Here\'s a different style of quote.</p>
<!-- /wp:paragraph -->

<!-- wp:pullquote -->
<figure class="wp-block-pullquote"><blockquote><p>Something I said in the article that I want to emphasize!</p><cite>Me</cite></blockquote></figure>
<!-- /wp:pullquote -->',
			'searchExclude' => true,
			'thumbnail' => $sample_featured_image
		),
	    array(
            'title' => 'Blog Update',
            'slug' => 'blog-update',
            'type' => 'es_template',
			'meta_input' => array( 'es_template_type' => 'post_notification' ),
            'content' => '<style>
.notice {background-color: #FFFFCC; border: 2px solid #AAAA88; font-size: 80%; padding: 10px; }
td { vertical-align: top; }
img { float: left; margin-right: 18px; width: inherit !important; height: inherit !important;max-width: inherit !important;}
div.mailposttitle { border: 1px black solid; margin-left: 1.3em; padding: 0.2em 0.8em; background-color: white; display: inline; z-index: 99; position: relative; border-radius: 30px; }
div.mailpost { background-color: #dddddd; border: 1px solid #808080; border-radius: 9px; padding: 12px; overflow: hidden; position: relative; top: -8px;}
</style>

Dear {{NAME}},

<p class="notice">You received this email because you signed up for blog updates on <a href="{{SITEURL}}">{{SITENAME}}</a>. If you no longer wish to receive these, use the Unsubscribe link at the end of this message.</p>

<div class=\'mailposttitle\'>New Post on {{SITENAME}}</div>
<div class=\'mailpost\'>
<h2>{{POSTLINK-WITHTITLE}}</h2>
<em>{{POSTCATS}}</em>

{{POSTIMAGE}}{{POSTEXCERPT}}
</div>'
            ),
    );

    $bCreatedPage = false;
    foreach ($default_pages as $new_page) 
    {
        $posttype = ($new_page['type'] == '' ? 'page' : $new_page['type']);
        $postobj = get_page_by_path( $new_page['slug'], '', $posttype );
        $content = str_replace("<lastid>", $lastid, $new_page['content']);
        if( $postobj === null )
        {
            // no entity of the required type and slug name was found. Create it.
            $add_default_pages = array(
                'post_title' => $new_page['title'],
                'post_name' => $new_page['slug'],
                'post_content' => $content,
                'post_status' => 'publish',
                'post_type' => $posttype
              );
			if (!empty($new_page['meta_input'])) {
				$add_default_pages['meta_input'] = $new_page['meta_input'];
			}
            // insert the post into the database
            $lastid = wp_insert_post($add_default_pages);
			printf('<p>' . esc_html__('Created %s: %s') . '</p>', $posttype, $new_page['title']);
			if ($newPage['searchExclude'] && !empty($pluginSearchExclude)) {
				$pluginSearchExclude->savePostIdToSearchExclude($lastid, true);
			}
			if (!empty($new_page['thumbnail'])) {
				set_post_thumbnail( $lastid, $new_page['thumbnail'] );
			}
			
            if (strchr($content, '<ownid>') !== false) {
            	// the content contains its own page ID, why do they do this? Now that we know this value, update the page accordingly.
            	$my_post = array('ID' => $lastid, 'post_content' => str_replace("<ownid>", $lastid, $content));
            	wp_update_post( $my_post );
            }
            // if metadata was specified, add that also.
            if ($new_page['meta'] != "") {
                $tags = explode(',', $new_page['meta']);
                foreach ($tags as $meta_assign) {
                    $parts = explode('=', $meta_assign, 2);
                    add_post_meta($lastid, $parts[0], $parts[1]);
                }
            }
            $bCreatedPage = true;
        } else {
            $lastid = $postobj->ID;
        }
        $new_page['id'] = $lastid;
    }
    if (!$bCreatedPage) {
	printf('<p>%s</p>', esc_html__('Did not need to create any pages.'));
    }
}

function torknado_author_create_menus() {
    // create menus
    $menu = torknado_create_menu_at_location('primary', __('Main Menu'), 'main-menu');
    $menu_items = array(
        array('menu-item-title' => __('Home'), 'menu-item-classes' => 'home', 'menu-item-url' => home_url( '/' )),
        array('menu-item-title' => __('About'), 'menu-item-object' => 'page', 'menu-item-object-id' => get_page_by_path('about')->ID, 'menu-item-type' => 'post_type'),
        array('menu-item-title' => __('Contact'), 'menu-item-object' => 'page', 'menu-item-object-id' => get_page_by_path('contact')->ID, 'menu-item-type' => 'post_type'),
        array('menu-item-title' => __('Blog'), 'menu-item-object' => 'page', 'menu-item-object-id' => get_page_by_path('blog')->ID, 'menu-item-type' => 'post_type'),
		array('menu-item-title' => __('Styling Test'), 'menu-item-object' => 'page', 'menu-item-object-id' => get_page_by_path('styling-test')->ID, 'menu-item-type' => 'post_type'),
    );
    if (torknado_populate_menu($menu, $menu_items)) {
    	printf('<p>%s</p>', esc_html(sprintf(__('Added items to menu %s.'), $menu->menu_name)));
    }
    
    /* $menu = torknado_create_menu_at_location('social', __('Social Links'), 'social-links');
    $menu_items = array(
        array('menu-item-title' => 'Facebook', 'menu-item-url' => 'https://www.facebook.com/tyler.tork.5'),
        array('menu-item-title' => 'Twitter', 'menu-item-url' => 'https://twitter.com/Tyler_Tork'),
    ); 
    if (torknado_populate_menu($menu, $menu_items)) {
    	printf('<p>%s</p>', esc_html(sprintf(__('Added items to menu %s.'), $menu->menu_name)));
    } */
}

function torknado_author_setup_site() {
    echo '<h2>' . esc_html__('Results of setup:') . '</h2><div class="updated">';
    torknado_author_setup_media_settings();
    torknado_author_setup_share_options();    
    torknado_author_create_pages();
    torknado_author_create_menus();
	torknado_create_igcampaign();
    torknado_post_setup_options();
	torknado_set_option('default_ping_status', 'closed', 'Disallowed trackbacks and pingbacks.');
	
    // assign the landing page and blog page.
    $page = get_page_by_path('books');
	$sof = update_option('show_on_front',  'page');
	$pof = update_option('page_on_front',  $page->ID);
	if ($sof || $pof) printf('<p>%s</p>', sprintf(esc_html__('Made %s the landing page.'), '<strong>' . esc_html($page->title) . '</strong>'));
    $page = get_page_by_path('blog');
    if (update_option('page_for_posts',  $page->ID)) printf('<p>%s</p>', sprintf(esc_html__('Made %s the blog page.'), '<strong>' . esc_html($page->title) . '</strong>')); 
    
    if (torknado_author_populate_widgets()) {
	printf('<p>%s</p>', esc_html__('Updated sidebar widgets.'));
    }
    if ( is_plugin_active("jetpack/jetpack.php") ) {
	deactivate_plugins( "jetpack/jetpack.php" );
  	printf('<p>%s</p>', esc_html__('Deactivated Jetpack plugin.'));
    }
    echo '</div>';
}


function torknado_author_required_plugins_active() {
	$reqd = array(
			'mooberry-book-manager/mooberry-book-manager.php',
			'wpforms-lite/wpforms.php',
			'wp-stats-manager/wp-stats-manager.php',
			'gallery-block-lightbox/gallery-block-lightbox.php',
			'email-subscribers/email-subscribers.php'
		);
	foreach ($reqd as $path) {
		if (!is_plugin_active($path)) return false;
	}
	return true;
}

/**
 * Register the required and recommended plugins for this theme.
 */
function torknado_author_register_required_plugins() {
    /*
     * Array of plugin arrays. Required keys are name and slug.
     * If the source is NOT from the .org repo, then source is also required.
     */
    $plugins = array(
        // List of plugins from the WordPress Plugin Repository.
        /* array(
            'name'      => 'Contact Form 7',
            'slug'      => 'contact-form-7',
            'required'  => true,
            'force_activation'   => true
        ), */
		array('name'=>'WPForms Lite', 'slug'=>'wpforms-lite', 'required'=>true, 'force_activation'=>true,
			'comment'=>'To make a form people can use to email you without knowing your email address. Includes spam blocker, which you must activate.'),
        array(
            'name'=>'Search Exclude', 'slug'=>'search-exclude', 'required'=>true,
			'comment'=>'Mark a page to not show up in a search of the site. Good for contact form "thanks" pages and the like.'),
        array(
            'name'=>'Antispam Bee', 'slug'=>'antispam-bee',
			'comment'=>'Install this if you let the general public post comments on your site. More accurate than Akismet, better performing, and more private.'),
        array(
            'name'=>'Mooberry Book Manager', 'slug'=>'mooberry-book-manager', 'required'=>true, 'force_activation'=>true,
			'comment'=>'Online catalog of books by you, in case you already have some. With buy links, Amazon preview, ...'),
        array(
            'name'=>'UpdraftPlus - Backup/Restore', 'slug'=>'updraftplus', 'required'=>true,
			'comment'=>'Automatic scheduled website backups. Set it up to store on Google Drive.'),
        array(
            'name'=>'iThemes Security', 'slug'=>'better-wp-security', 'required'=>true,
			'comment'=>'WordPress is pretty secure already, but this does more to block hackers.'),
        array(
            'name'=>'Sassy Social Share','slug'=>'sassy-social-share',
			'comment'=>'Buttons to facilitate people sharing your content without invading their privacy. Includes "floating" buttons and follow links.'),
		array(
            'name'=>'Genesis Blocks', 'slug'=>'genesis-blocks',
			'comment'=>'Better responsive columns for page building, among other things.'),
        array(
            'name'=>'WordPress SEO by Yoast', 'slug'=>'wordpress-seo', 'is_callable'=>'wpseo_init', 'required'=>true, 'force_activation'=>true,
			'comment'=>'Improve your site\'s search ranking, have better control of Facebook and Twitter sharing, get advice about reading score.'),
        array(
            'name'=>'WP Mail SMTP', 'slug'=>'wp-mail-smtp', 'required'=>true,
			'comment'=>'Send all email with authentication to get past spam filters.'),
        array(
            'name'=>'WP Add Mime Types', 'slug'=>'wp-add-mime-types',
			'comment'=>'Use this to allow upload of sample ebooks (PDF is already allowed).'),
		array(
			'name'=>'WP Visitor Statistics (Real Time Traffic)', 'slug'=>'wp-stats-manager', 'required'=>true, 'force_activation'=>true,
			'comment'=>'Basic analytics with full privacy for your visitors. If you need more, try Google Analytics or Matomo.'
		),
		array(
			'name'=>'Lightbox for the Default Gallery & Image Block...', 'slug'=>'gallery-block-lightbox', 'required'=>true, 'force_activation'=>true,
			'comment'=>'Opens "click to enlarge" images in a lightbox instead of their own tab.'),
		array(
			'name'=>'Hide featured image on all single page/post', 'slug'=>'hide-featured-image-on-all-single-pagepost', 'required'=>true,
			'comment'=>'Prevents theme from displaying featured image on posts. Depending which theme you use, this might not be needed.'),
		array(
			'name'=>'Hide Comment Author Link', 'slug'=>'hide-wp-comment-author-link',
			'comment'=>'Only needed if you plan to allow comments from the general public, hides the "Web Site" field spammers love.'),
		array(
			'name'=>'Email Subscribers & Newsletters', 'slug'=>'email-subscribers', 'required'=>true, 'force_activation'=>true,
			'comment'=>'Manage your mailing list and send automatic blog updates entirely on your own site. You might prefer Mailchimp if you have a larger list.'
		),

    );
    
    $config = array(
        'id'           => 'torknado-author',       // Unique ID for hashing notices for multiple instances of TGMPA.
        'default_path' => '',                      // Default absolute path to bundled plugins.
        'menu'         => 'tgmpa-install-plugins', // Menu slug.
        'parent_slug'  => 'themes.php',            // Parent menu slug.
        'capability'   => 'edit_theme_options',    // Capability needed to view plugin install page, should be a capability associated with the parent menu used.
        'has_notices'  => true,                    // Show admin notices or not.
        'dismissable'  => false,                    // If false, a user cannot dismiss the nag message.
		'pester_to_activate' => false,			   // if false, banner is not displayed if there are only things to activate, not install.
        'dismiss_msg'  => 'This message will go away when you remove the Torknado Author plugin.', // If not 'dismissable', this message shown at top of nag.
        'is_automatic' => true,                    // Automatically activate plugins after installation or not.
        'message'      => '<p>' . __('Plugins you may want for your author website. Read comments column to decide whether you need them.') . '</p><p>' . sprintf(__('After installing required plugins, click %s in dashboard to continue setup.'), torknado_author_plugin_resume_link(true)) . '</p>',                      // Message to output right before the plugins table.
		'strings'	   => array(
			'page_title' => __('Install Recommended Plugins'),
			'notice_can_install_required'     => _n_noop('Automatic setup requires this plugin: %1$s.',
					'Automatic setup requires these plugins: %1$s.', 'tnado-auth'),
			'notice_can_install_recommended'  => _n_noop(
					'You might also like this plugin: %1$s.', 'You might also like these plugins: %1$s.', 'tnado-auth'),
			'notice_can_activate_required'    => _n_noop(
					'This plugin is installed but not active (which is fine, just FYI): %1$s.',
					'These plugins are installed but not active (which is fine, just FYI): %1$s.'),
			'complete' => torknado_author_plugin_resume_link(),
			'dashboard' => torknado_author_plugin_resume_link(),
			'skin_upgrade_start' => __('The installation and activation process is starting. <strong>This may take a while</strong>.') . '</p><p>' .
				__('When finished, click ') . torknado_author_plugin_resume_link(true),
			'notice_can_activate_recommended' => '', // hide this notice.
		)
    );

    tgmpa( $plugins, $config );
}

function torknado_author_plugin_resume_link($mini = false) {
	if ($mini) {
		return '<a href="/wp-admin/tools.php?page=torknado-author" style=\'color:white;background-color:#23282d; display:inline-block; padding:4px\'>' . __('Author Site Setup') . '</a>';
	} else {
		return ('<a href="/wp-admin/tools.php?page=torknado-author" style="	font-size: 24px; background-color: pink; border: 2px solid darkred; border-radius: 20px; padding: 10px; text-decoration: none; margin-top: 13px; display: inline-block">' . esc_html__( 'Resume Author Website setup', 'tnado-auth' ) . '</a>');
	}
}

function torknado_author_add_plugin_page_link( $links ) {
	// add a "setup" button in the Plugins page, below the title of this plugin.
	$links[] = '<a class="button-primary" href="' .
		admin_url( 'admin.php?page=torknado-author' ) .
		'">' . __('Setup') . '</a>';
	return $links;
}

function torknado_author_activation_redirect( $plugin ) {
    // called after plugin is first activated. Steer user to dashboard, where they'll see the big setup button.
    if( $plugin == plugin_basename( __FILE__ ) ) {
        exit( wp_redirect( admin_url( 'admin.php?page=torknado-author' ) ) );
    }
}

function torknado_author_activation() {
// what to do when the plugin is first activated during a session-- maybe nothing.
	torknado_set_preinstall_options();	// we're about to install and activate some plugins. Prevent them pestering the user with wizards and alerts.
}

function torknado_author_add_to_dash() {
// called when the dashboard is being displayed, this shows our widget with the big setup button, at the top of the dash.
    torknado_add_dashboard_widget( 'torknado-auth-dash', __('Torknado Author Website Setup'), 'torknado_author_show_dash_widget', 'torknado_author_controls' );
}

function torknado_create_widget($widget_name, $widget_data) {
// create a new widget and return the name to insert in sidebar_widgets option.
    $widget_instances = get_option( 'widget_' . $widget_name, array() );
    $numeric_keys = array_filter( array_keys( $widget_instances ), 'is_int' );
    $next_key = $numeric_keys ? max( $numeric_keys ) + 1 : 2;
    $new_slug = $widget_name . '-' . $next_key;
    $widget_instances[$next_key] = $widget_data ? $widget_data : array();
    update_option('widget_' . $widget_name, $widget_instances);
    //@@echo '<p>ADD WIDGET: ' . $widget_name . ' => ' . (serialize($widget_instances)) . '</p>';
    return $new_slug;
}

/**
 * Determine the position of a given widget within a sidebar.
 * @param array $sidebar  An element of the sidebar_widgets option.
 * @param string $widget_name  E.g. "search"
 * @returns false if not found, else zero-based index.
 */
function torknado_find_widget_in_sidebar($sidebar, $widget_name) {
    for ($ind = 0; $ind < count($sidebar); $ind++) {
	if ($widget_name == torknado_strleftback($sidebar[$ind], '-')) return $ind;
    }
    return false;
}

/**
 * Insert a list of widgets into a sidebar, at the beginning of the sidebar.
 * 
 * @param string $area_id   ID of the widget area.
 * @param array $new-widgets	Each element has the widget ID as key, and the widget data as variable, e.g. 'search'=>array('title'=>'')
 */
function torknado_insert_widgets($area_id, $new_widgets) {
    $sidebars = get_option('sidebars_widgets'); // get the list of sidebars
    $sidebar = $sidebars[$area_id];
	$added = '';
    $bSave = false;
    if (!isset($sidebar)) {
    	$sidebar = array();
    }
    $ind = 0;
    if (!empty($new_widgets)) {
    	foreach ($new_widgets as $widgName => $widgData) {
    	    //@@echo '<p>INPUT: ' . $widgName . ' / ' . (serialize($widgData)) . '</p>';
    	    $bMod = $false;
    	    $pos = torknado_find_widget_in_sidebar($sidebar, $widgName);
    	    if ($pos === false) {
    	    	// it's not already in the sidebar -- create one and add it.
    	    	//@@echo '<p>NOT FOUND: ' . $widgName . '</p>';
    	    	$slug = torknado_create_widget($widgName, $widgData);
    	    	$bMod = true;
    	    } else if ($pos != $ind) { // move the widget to the right position.
    	    	//@@echo '<p>FOUND AT POS ' . $pos . ': ' . $widgName . '</p>';
    	    	$bMod = true;
    	    	$slug = $sidebar[$pos];
    	    	// remove the item from the wrong position.
		array_splice($sidebar, $pos, 1);
		//@@echo '<p>AFTER REMOVE: ' . serialize($sidebar) . '</p>';
    	    }
    	   //@@  else { echo '<p>Already at position ' . $ind . '</p>'; }
    	    
    	    if ($bMod) {
				$added .= ($added == "" ? "" : ", ") . $widgName;
    	        array_splice($sidebar, $ind, 0, $slug);
    	        //@@echo '<p>AFTER ADD: ' . serialize($sidebar) . '</p>';
    	        $bSave = true;
    	    }
    	    $ind++;
    	}
    }
    
    if ($bSave) {
    	$sidebars[$area_id] = $sidebar;
		printf("<p>%s</p>", esc_html__("Added sidebar widgets: ") . $added);
    	//@@echo '<p>UPDATE SIDEBARS: ' . (serialize($sidebars)) . '</p>';
    	update_option( 'sidebars_widgets', $sidebars );
    }
    return $bSave;
}

function torknado_get_primary_sidebar() {
   // returns ID of theme's main sidebar (best guess), or empty string if none found.
    $sidebars = get_option('sidebars_widgets'); // get the list of sidebars
    // if one has an id containing "primary" or that is named sidebar-1, that's the one we want.
    foreach ($sidebars as $key => $sidebar) {
		if (strpos($key, 'primary') !== false or $key == 'sidebar-1') return $key;
    }
    // otherwise just take the first one in the list.
    foreach ($sidebars as $key => $sidebar) {
	if ($key != 'wp_inactive_widgets') {
	    return $key;
	}
    }
}

function torknado_author_populate_widgets() {
    $desired_widgets = array(
    	'search' => array ('title' => ''),
		'custom_html' => array('title' => '', 'content' => '<a href=\'/subscribe\' style=\'display: inline-block; border: 1px solid gray; border-radius: 6px; background-color: #cccccc; padding: 7px; text-decoration: none; font-weight: bold\'>Subscribe</a> for blog updates'),
    	'recent-posts' => array ( 'title' => '', 'number' => 5),
    	'categories' => array ( 'title' => '', 'count' => 0, 'hierarchical' => 0, 'dropdown' => 0 )
    );
    $primary_id = torknado_get_primary_sidebar();
    if ($primary_id != '') {
    	return torknado_insert_widgets($primary_id, $desired_widgets);
    } else {
        echo '<p>' . esc_html__("Can't find any widget areas in this theme -- skipping widget configuration.") . '</p>';
    }
    return false;
}

function torknado_author_debug() {
//$postobj = get_page_by_path( 'form2', '', 'wpforms' );
//echo '<p>ID=' . $postobj->ID . '<br>' . esc_html($postobj->post_content) . '</p>';
//$meta = get_post_meta( $postobj->ID);
//echo '<p>' . serialize($meta) . '</p>';
$locs = get_registered_nav_menus();
echo var_export($locs);
}

//global $torknado_debug = false;

register_activation_hook( __FILE__, 'torknado_author_activation' );
add_action( 'wp_dashboard_setup', 'torknado_author_add_to_dash' );
add_action( 'activated_plugin', 'torknado_author_activation_redirect' );
add_action( 'admin_menu', 'torknado_author_plugin_menu' );
add_action( 'admin_enqueue_scripts', 'torknado_load_admin_scripts' );
require_once dirname( __FILE__ ) . '/inc/class-tgm-plugin-activation.php';
add_action( 'tgmpa_register', 'torknado_author_register_required_plugins' );
add_filter('plugin_action_links_'.plugin_basename(__FILE__), 'torknado_author_add_plugin_page_link');
?>